@@ -1,10 +1,10 @@
 '''Write snippets of code in C++, Python, Ruby, and others as documentation and execute them as regression tests.'''
 
-__version__ = "8.0.1"
+__version__ = "9.1.1"
 
-_author  = 'Di Paola Martin'
+_author = 'Di Paola Martin'
 _license = 'GNU GPLv3'
-_url = 'https://github.com/byexamples/byexample'
+_url = 'https://byexamples.github.io'
 
 _license_disclaimer = r'''Copyright (C) {author} - {url}
 
