@@ -1,115 +1,37 @@
 from __future__ import unicode_literals
-import collections, re, os
-from .common import log, build_where_msg, tohuman, \
-                    enhance_exceptions, print_example, constant
+import re, os
+from .common import build_where_msg, tohuman, \
+                    enhance_exceptions
 
 from .parser import ExampleParser
 from .options import Options
+from .log import clog, log_context, DEBUG, CHAT, log_with
 
-class Where(object):
-    def __init__(self, start_lineno, end_lineno, filepath, zdelimiter):
-        self.start_lineno = start_lineno
-        self.end_lineno = end_lineno
-        self.filepath = filepath
-        self.zdelimiter = zdelimiter
+from .example import Where, Zone, Example
+'''
+>>> from byexample.log import init_log_system
+>>> init_log_system()
+'''
 
-    def as_tuple(self):
-        return (self.start_lineno, self.end_lineno, self.filepath, self.zdelimiter)
 
-    def __iter__(self):
-        return iter(self.as_tuple())
+def _build_fake_example(
+    snippet,
+    expected,
+    language='python',
+    start_lineno=0,
+    specific=False,
+    fully_parsed=True,
+    opts=None
+):
+    class R:
+        pass  # <- fake runner instance
 
-    def __repr__(self):
-        return repr(self.as_tuple())
+    R.language = language  # <- language of the example
 
-class Zone(object):
-    def __init__(self, zdelimiter, zone_str, where):
-        self.zdelimiter = zdelimiter
-        self.str = zone_str
-        self.where = where
+    class F:
+        pass  # <- fake finder instance
 
-class Example(object):
-    '''
-    The unit work of byexample: the example.
-
-    It represents a example found in <where> by the <finder> that should be
-    parsed by the <parser> and executed later by the <runnner>.
-
-    The piece of text found is given by <snippet> and <extracted_str>:
-    the code the be executed and the expected output.
-
-    These two are *incomplete* and further processing is need by <parser>
-
-    >>> from byexample.finder import _build_fake_example
-
-    >>> example = _build_fake_example('f()', '42', fully_parsed=False)
-    >>> example
-    Example (not parsed yet) [python] in file file.md, lines 0-2
-
-    The example is incomplete or not fully parsed because the <snippet> may
-    not be the code in its final state; the expected regex doesn't exist nor
-    the options.
-
-    >>> example.source
-    <...>
-    AttributeError: 'Example' object has no attribute 'source'
-
-    >>> example.expected
-    <...>
-    AttributeError: 'Example' object has no attribute 'expected'
-
-    >>> example.options
-    <...>
-    AttributeError: 'Example' object has no attribute 'options'
-
-    After the completion those attributes should be defined
-
-    >>> example.parse_yourself()
-    Example [python] in file file.md, lines 0-2
-
-    >>> example.source
-    'f()\n'
-
-    >>> example.expected.str
-    '42'
-
-    >>> example.options
-    {'norm_ws': False, 'rm': [], 'tags': True}
-
-    '''
-    def __init__(self, finder, runner, parser, snippet, expected_str, indent, where):
-        self.finder, self.runner, self.parser = finder, runner, parser
-        self.snippet, self.expected_str, self.indentation = snippet, expected_str, indent
-
-        self.start_lineno, self.end_lineno, self.filepath, self.zdelimiter = where
-
-        self.fully_parsed = False
-
-    def parse_yourself(self, concerns=None):
-        if self.fully_parsed:
-            raise ValueError("You cannot parse/build an example twice: " + \
-                             repr(self))
-
-        where = Where(self.start_lineno, self.end_lineno, self.filepath, self.zdelimiter)
-        self.parser.parse(self, concerns)
-        self.fully_parsed = True
-
-        return self
-
-    def __repr__(self):
-        f = "" if self.fully_parsed else "(not parsed yet) "
-        return "Example %s[%s] in file %s, lines %i-%i" % (
-                        f,
-                        self.runner.language,
-                        self.filepath, self.start_lineno, self.end_lineno)
-
-def _build_fake_example(snippet, expected, language='python', start_lineno=0,
-                            specific=False, fully_parsed=True, opts=None):
-    class R: pass    # <- fake runner instance
-    R.language = language # <- language of the example
-
-    class F: pass    # <- fake finder instance
-    F.specific = specific # <- is finder specific?
+    F.specific = specific  # <- is finder specific?
 
     # fake a parser
     parser = ExampleParser(0, 'utf8', Options())
@@ -117,7 +39,15 @@ def _build_fake_example(snippet, expected, language='python', start_lineno=0,
 
     # fake the options parsed by the parser
     if opts == None:
-        opts = {'norm_ws': False, 'tags': True, 'rm': []}
+        opts = Options(
+            {
+                'norm_ws': False,
+                'tags': True,
+                'rm': [],
+                'input': False,
+                'input_prefix_range': (6, 12)
+            }
+        )
     parser.extract_options = lambda x: opts
 
     # fake the start-end lines where the example "was found"
@@ -134,6 +64,7 @@ def _build_fake_example(snippet, expected, language='python', start_lineno=0,
 
     return e
 
+
 class ExampleHarvest(object):
     '''
                   Finding process             Parsing process
@@ -160,8 +91,10 @@ class ExampleHarvest(object):
                                       \       reporter
                                        \         .
     '''
-    def __init__(self, allowed_languages, registry, verbosity,
-                        options, use_colors, encoding, **unused):
+    def __init__(
+        self, allowed_languages, registry, verbosity, options, use_colors,
+        encoding, **unused
+    ):
         self.allowed_languages = allowed_languages
         self.verbosity = verbosity
         self.use_colors = use_colors
@@ -178,64 +111,68 @@ class ExampleHarvest(object):
         return 'Example Harvester'
 
     def get_examples_from_file(self, filepath):
-        try:
-            f = open(filepath, 'rtU', encoding=self.encoding)
-            already_encoded = True
-        except TypeError:
-            # Python 2.7
-            f = open(filepath, 'rbU')
-            already_encoded = False
+        f = open(filepath, 'rtU', encoding=self.encoding)
 
         with f as f:
             string = f.read()
-            if not already_encoded:
-                string = string.decode(self.encoding)
 
         return self.get_examples_from_string(string, filepath)
 
-    def get_examples_from_string(self, string, filepath='<string>'):
-        all_examples = []
+    @log_context('byexample.zones')
+    def _get_zones(self, string, filepath='<string>'):
         _, ext = os.path.splitext(filepath)
 
-        log("Finding examples...", self.verbosity-1)
         zdelimiter = self.zdelimiter_by_file_extension.get(
-                ext,
-                self.zdelimiter_by_file_extension['no-delimiter']
-                )
+            ext, self.zdelimiter_by_file_extension['no-delimiter']
+        )
         zones = self.get_zones_using(
-                    zdelimiter,
-                    string,
-                    filepath,
-                    start_lineno=1)
+            zdelimiter, string, filepath, start_lineno=1
+        )
 
-        log("File '%s': %i zones [%s]" % (filepath, len(zones),
-                                        str(zdelimiter)), self.verbosity-2)
+        clog().chat(
+            "File '%s': %i zones [%s]", filepath, len(zones), str(zdelimiter)
+        )
 
-        if self.verbosity-3 >= 0:
+        if clog().isEnabledFor(DEBUG):
             for zone in zones:
-                log("Zone %s" % (zone.where), self.verbosity-3)
+                clog().debug("Zone %s", zone.where)
+
+        return zones
+
+    @log_context('byexample.find')
+    def get_examples_from_string(self, string, filepath='<string>'):
+        all_examples = []
+        zones = self._get_zones(string, filepath)
 
         for finder in self.available_finders:
             nexamples = 0
             for zone in zones:
                 examples = self.get_examples_using(
-                        finder,
-                        zone.str,
-                        zone.where.filepath,
-                        zone.where.start_lineno)
+                    finder, zone.str, zone.where.filepath,
+                    zone.where.start_lineno
+                )
                 all_examples.extend(examples)
                 nexamples += len(examples)
 
-            log("File '%s': %i examples [%s]" % (filepath, nexamples,
-                                            str(finder)), self.verbosity-2)
+            clog().chat(
+                "File '%s': %i examples [%s]", filepath, nexamples,
+                str(finder)
+            )
 
         # sort the examples in the same order
         # that they were found in the file/string;
         # see check_example_overlap
-        all_examples.sort(key=lambda this: (this.start_lineno, -this.end_lineno))
+        all_examples.sort(
+            key=lambda this: (this.start_lineno, -this.end_lineno)
+        )
 
         all_examples = self.check_example_overlap(all_examples, filepath)
 
+        tmp = set(e.runner.language for e in all_examples)
+        clog().chat(
+            "Findings in file '%s': %i examples written in %i different languages in %i zones were found.",
+            filepath, len(all_examples), len(tmp), len(zones)
+        )
         return all_examples
 
     def check_example_overlap(self, examples, filepath):
@@ -306,7 +243,7 @@ class ExampleHarvest(object):
         while not collision_free:
             collision_free = True
             if not examples:
-                return examples # pragma: no cover
+                return examples  # pragma: no cover
 
             prev = examples[0]
             for i, example in enumerate(examples[1:], 1):
@@ -323,16 +260,22 @@ class ExampleHarvest(object):
                     continue
 
                 collision_free = not any_collision
-                curr_where = Where(example.start_lineno, example.end_lineno, filepath, example.zdelimiter)
-                prev_where = Where(prev.start_lineno, prev.end_lineno, filepath, prev.zdelimiter)
+                curr_where = Where(
+                    example.start_lineno, example.end_lineno, filepath,
+                    example.zdelimiter
+                )
+                prev_where = Where(
+                    prev.start_lineno, prev.end_lineno, filepath,
+                    prev.zdelimiter
+                )
 
                 self._log_debug(" * Collision Type (1/2/3): %s/%s/%s\n"        \
                                 " * Languages (prev/current): %s/%s\n"         \
                                     % (collision_type_1, collision_type_2,
                                         collision_type_3, prev.runner.language,
                                         example.runner.language), curr_where)
-                print_example(prev, self.use_colors, self.verbosity-3)
-                print_example(example, self.use_colors, self.verbosity-3)
+                prev.pretty_print()
+                example.pretty_print()
 
                 if collision_type_2:
                     del examples[i]
@@ -341,51 +284,43 @@ class ExampleHarvest(object):
 
                 msg = "In %s, examples at lines %i-%i (found by %s) and " +\
                       "at lines %i-%i (found by %s) overlap each other."
-                msg = msg % (filepath, example.start_lineno, example.end_lineno,
-                             example.finder, prev.start_lineno, prev.end_lineno,
-                             prev.finder)
+                msg = msg % (
+                    filepath, example.start_lineno, example.end_lineno,
+                    example.finder, prev.start_lineno, prev.end_lineno,
+                    prev.finder
+                )
                 raise ValueError(msg)
 
-        if self.verbosity-1 >= 0:
-            log("Examples after removing any overlapping", self.verbosity-1)
+        if clog().isEnabledFor(CHAT):
+            clog().debug("Examples after removing any overlapping")
             for finder in set(e.finder for e in examples):
-                log("File '%s': %i examples [%s]" % (
-                                    filepath,
-                                    len([e for e in examples if e.finder==finder]),
-                                    str(finder)),
-                                    self.verbosity-1)
+                clog().chat(
+                    "File '%s': %i examples [%s]", filepath,
+                    len([e for e in examples if e.finder == finder]),
+                    str(finder)
+                )
 
-        if self.verbosity-2 >= 0:
+        if True or clog().isEnabledFor(DEBUG):
             for e in examples:
-                print_example(e, self.use_colors, 0)
-            print("")
+                e.pretty_print()
         return examples
 
     def _log_drop(self, reason, where):
         self._log_debug(" => Dropped example: " + reason, where)
 
     def _log_debug(self, what, where):
-        log(build_where_msg(where, self, what), self.verbosity-3)
+        clog().debug(build_where_msg(where, self, what))
 
     def get_examples_using(self, finder, string, filepath, start_lineno):
         return self.from_string_get_items_using(
-                finder,
-                string,
-                self.get_example,
-                'examples',
-                filepath,
-                start_lineno
-                )
+            finder, string, self.get_example, 'examples', filepath,
+            start_lineno
+        )
 
     def get_zones_using(self, zdelimiter, string, filepath, start_lineno):
         return self.from_string_get_items_using(
-                zdelimiter,
-                string,
-                self.get_zone,
-                'zones',
-                filepath,
-                start_lineno
-                )
+            zdelimiter, string, self.get_zone, 'zones', filepath, start_lineno
+        )
 
     def get_example(self, finder, match, where):
         with enhance_exceptions(where, finder):
@@ -400,17 +335,22 @@ class ExampleHarvest(object):
                 self._log_drop('language %s not allowed' % language, where)
                 return
 
+        with enhance_exceptions(where, finder), log_with(language):
             # who can parse it?
             parser = self.parser_by_language.get(language)
             if not parser:
-                self._log_drop('no parser found for %s language' % language, where)
-                return # TODO should be an error?
+                self._log_drop(
+                    'no parser found for %s language' % language, where
+                )
+                return  # TODO should be an error?
 
             # who can execute it?
             runner = self.runner_by_language.get(language)
             if not runner:
-                self._log_drop('no runner found for %s language' % language, where)
-                return # TODO should be an error?
+                self._log_drop(
+                    'no runner found for %s language' % language, where
+                )
+                return  # TODO should be an error?
 
             # save the indentation here
             indent = match.group('indent')
@@ -421,10 +361,11 @@ class ExampleHarvest(object):
             if expected == None:
                 expected = ""
 
-        with enhance_exceptions(where, parser):
+        with enhance_exceptions(where, parser), log_with(language):
             # perfect, we have everything to build an example
-            example = Example(finder, runner, parser,
-                                    snippet, expected, indent, where)
+            example = Example(
+                finder, runner, parser, snippet, expected, indent, where
+            )
             return example
 
     def get_zone(self, zdelimiter, match, where):
@@ -432,8 +373,16 @@ class ExampleHarvest(object):
             zone_str = zdelimiter.get_zone(match, where)
             return Zone(zdelimiter, zone_str, where)
 
-    def from_string_get_items_using(self, matcher, string, getter, what,
-            filepath='<string>', start_lineno=1, zdelimiter=None):
+    def from_string_get_items_using(
+        self,
+        matcher,
+        string,
+        getter,
+        what,
+        filepath='<string>',
+        start_lineno=1,
+        zdelimiter=None
+    ):
         charno = 0
         items = []
 
@@ -457,6 +406,7 @@ class ExampleHarvest(object):
                 items.append(item)
         return items
 
+
 class ExampleFinder(object):
     specific = True
 
@@ -465,16 +415,16 @@ class ExampleFinder(object):
         self.encoding = encoding
 
     def example_regex(self):
-        raise NotImplementedError() # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
     def get_matches(self, string):
         return self.example_regex().finditer(string)
 
     def get_language_of(self, options, match, where):
-        raise NotImplementedError() # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
     def __repr__(self):
-        return '%s Finder' % tohuman(self.target)
+        return '%s Finder' % tohuman(self.target if self.target else self)
 
     def check_and_remove_indent(self, example_str, indent, where):
         r'''
@@ -616,7 +566,7 @@ class ZoneDelimiter(object):
         self.encoding = encoding
 
     def zone_regex(self):
-        raise NotImplementedError() # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
     def get_matches(self, string):
         return self.zone_regex().finditer(string)
@@ -625,4 +575,6 @@ class ZoneDelimiter(object):
         return match.group('zone')
 
     def __repr__(self):
-        return '%s Zone Delimiter' % tohuman(self.target)
+        return '%s Zone Delimiter' % tohuman(
+            self.target if self.target else self
+        )
