@@ -8,7 +8,9 @@ from .executor import FileExecutor
 from .differ import Differ
 from .parser import ExampleParser
 from .concern import Concern, ConcernComposite
-from .common import log
+from .common import enhance_exceptions
+from .log import clog, log_context, configure_log_system, setLogLevels, TRACE, DEBUG, CHAT, INFO, NOTE, ERROR, CRITICAL
+
 
 def are_tty_colors_supported(output):
     def get_colors():
@@ -35,6 +37,7 @@ def are_tty_colors_supported(output):
            'm' not in os.getenv('TERM', '').split('-') and \
            get_colors() >= 8
 
+
 def is_a(target_class, key_attr, warn_missing_key_attr):
     '''
     Returns a function that will return True if its argument
@@ -54,47 +57,56 @@ def is_a(target_class, key_attr, warn_missing_key_attr):
         attr_ok = hasattr(obj, key_attr)
 
         if class_ok and not attr_ok:
-            log(" * Warning: class '%s' has not attribute '%s'." % \
-                    (obj.__name__, key_attr),  0)
+            clog().warn(
+                "Class '%s' has not attribute '%s'.", obj.__name__, key_attr
+            )
 
         return class_ok and attr_ok
 
     return _is_X
 
+
+@log_context('byexample.load')
 def load_modules(dirnames, cfg):
     verbosity = cfg['verbosity']
-    registry = {'runners': {},
-                'finders': {},
-                'parsers': {},
-                'concerns': {},
-                'zdelimiters': {},
-                }
+    registry = {
+        'runners': {},
+        'finders': {},
+        'parsers': {},
+        'concerns': {},
+        'zdelimiters': {},
+    }
     for importer, name, is_pkg in pkgutil.iter_modules(dirnames):
         path = importer.path
 
-        log("From '%s' loading '%s'..." % (path, name), verbosity-2)
+        clog().debug("From '%s' loading '%s'...", path, name)
 
         try:
             module = importer.find_module(name).load_module(name)
         except Exception as e:
-            log("From '%s' loading '%s'...failed: %s" % (path, name, str(e)),
-                                                        verbosity)
+            clog().info(
+                "From '%s' loading '%s'...failed: %s", path, name, str(e)
+            )
             continue
 
         stability = getattr(module, 'stability', 'undefined')
-        if stability not in ('experimental', 'provisional', 'unstable', 'stable', 'deprecated'):
+        if stability not in (
+            'experimental', 'provisional', 'unstable', 'stable', 'deprecated'
+        ):
             stability = 'experimental/%s?' % str(stability)
 
-        log("From '%s' loaded '%s' (%s)" % (path, name, stability), verbosity-1)
-        for klass, key, what in [(ExampleRunner, 'language', 'runners'),
-                                 (ExampleParser, 'language', 'parsers'),
-                                 (ExampleFinder, 'target', 'finders'),
-                                 (ZoneDelimiter, 'target', 'zdelimiters'),
-                                 (Concern, 'target', 'concerns')]:
+        clog().chat("From '%s' loaded '%s' (%s)", path, name, stability)
+        for klass, key, what in [
+            (ExampleRunner, 'language', 'runners'),
+            (ExampleParser, 'language', 'parsers'),
+            (ExampleFinder, 'target', 'finders'),
+            (ZoneDelimiter, 'target', 'zdelimiters'),
+            (Concern, 'target', 'concerns')
+        ]:
 
             # we are interested in any class that is a subclass of 'klass'
             # and it has an attribute 'key'
-            predicate = is_a(klass, key, verbosity-2)
+            predicate = is_a(klass, key, verbosity - 2)
 
             container = registry[what]
             klasses_found = inspect.getmembers(module, predicate)
@@ -104,6 +116,12 @@ def load_modules(dirnames, cfg):
                 # remove already loaded
                 klasses_found = set(klasses_found) - set(container.values())
 
+            if klasses_found:
+                clog().debug(
+                    "Classes found for '%s': %s", what,
+                    ', '.join(k.__name__ for k in klasses_found)
+                )
+
             objs = [klass(**cfg) for klass in klasses_found]
             if objs:
                 loaded_objs = []
@@ -117,10 +135,15 @@ def load_modules(dirnames, cfg):
                             container[k] = obj
                         loaded_objs.append(obj)
 
-                log("\n".join((" - %s" % repr(i)) for i in loaded_objs), verbosity-1)
+                clog().chat(
+                    "\n".join((" - %s" % repr(i)) for i in loaded_objs)
+                )
+            else:
+                clog().chat("No classes found for '%s'.", what)
 
     return registry
 
+
 def get_allowed_languages(registry, selected):
     available = set([obj.language for obj in registry['runners'].values()] + \
                       [obj.language for obj in registry['parsers'].values()])
@@ -138,6 +161,7 @@ def get_allowed_languages(registry, selected):
                                not_found)
     return selected
 
+
 def verify_encodings(input_encoding, verbosity):
     if sys.stdout.encoding is None:
         try:
@@ -146,14 +170,17 @@ def verify_encodings(input_encoding, verbosity):
         except:
             e = 'UTF-8'
 
-        log(("The encoding of your terminal is unset.\n" +
-                "Try to set the environment variable PYTHONIOENCODING='%s' first\n" +
-                "or run 'byexample' like this:\n" +
-                "  PYTHONIOENCODING='%s' byexample ...\n") % (e, e), verbosity)
+        clog().error(
+            "The encoding of your terminal is unset.\n" +
+            "Try to set the environment variable PYTHONIOENCODING='%s' first\n"
+            + "or run 'byexample' like this:\n" +
+            "  PYTHONIOENCODING='%s' byexample ...\n", e, e
+        )
         sys.exit(-1)
 
-    log("Encoding (input): %s." % input_encoding, verbosity-2)
-    log("Encoding (output): %s." % sys.stdout.encoding, verbosity-2)
+    clog().chat("Encoding (input): %s.", input_encoding)
+    clog().chat("Encoding (output): %s.", sys.stdout.encoding)
+
 
 def geometry(x):
     lines, columns = [int(v.strip()) for v in str(x).split('x')]
@@ -162,76 +189,107 @@ def geometry(x):
 
     return (lines, columns)
 
+
+def _range(x):
+    min, max = [int(v.strip()) for v in str(x).split(':')]
+    if min < 0 or max < 0 or min > max:
+        raise ValueError("Invalid range %s" % x)
+
+    return (min, max)
+
+
 def get_default_options_parser(cmdline_args):
     options_parser = OptionParser()
     options_parser.add_flag(
-            "fail-fast",
-            default=cmdline_args.fail_fast,
-            help="if an example fails, fail and stop all the execution.")
+        "fail-fast",
+        default=cmdline_args.fail_fast,
+        help="if an example fails, fail and stop all the execution."
+    )
     options_parser.add_flag(
-            "norm-ws",
-            default=False,
-            help="ignore the amount of whitespaces.")
+        "norm-ws", default=False, help="ignore the amount of whitespaces."
+    )
     options_parser.add_flag(
-            "pass",
-            default=False,
-            help="run the example but do not check its output.")
+        "pass",
+        default=False,
+        help="run the example but do not check its output."
+    )
     options_parser.add_flag(
-            "skip",
-            default=False,
-            help="do not run the example.")
+        "skip", default=False, help="do not run the example."
+    )
     options_parser.add_flag(
-            "tags",
-            default=True,
-            help="enable the tags <...>.")
+        "tags", default=True, help="enable the tags <...>."
+    )
     options_parser.add_flag(
-            "enhance-diff",
-            default=cmdline_args.enhance_diff,
-            help="improve how the diff are shown.")
+        "enhance-diff",
+        default=cmdline_args.enhance_diff,
+        help="improve how the diff are shown."
+    )
     options_parser.add_argument(
-            "+rm",
-            default=[],
-            action='append',
-            help="remove a character from the got and expected strings.")
+        "+rm",
+        default=[],
+        action='append',
+        help="remove a character from the got and expected strings."
+    )
     options_parser.add_argument(
-            "+timeout",
-            default=cmdline_args.timeout,
-            type=float,
-            help="timeout in seconds to complete the example.")
+        "+timeout",
+        default=cmdline_args.timeout,
+        type=float,
+        help="timeout in seconds to complete the example."
+    )
     options_parser.add_argument(
-            "+diff",
-            default=cmdline_args.diff,
-            choices=['none', 'unified', 'ndiff', 'context'],
-            help="select diff algorithm.")
+        "+diff",
+        default=cmdline_args.diff,
+        choices=['none', 'unified', 'ndiff', 'context', 'tool'],
+        help="select diff algorithm."
+    )
     options_parser.add_argument(
-            "+geometry",
-            default=(24, 80),
-            type=geometry,
-            help="number of lines and columns of the terminal of the form LxC (default to 24x80).")
+        "+geometry",
+        default=(24, 80),
+        type=geometry,
+        help=
+        "number of lines and columns of the terminal of the form LxC (default to 24x80)."
+    )
     options_parser.add_argument(
-            "+term",
-            default='dumb',
-            choices=['as-is', 'dumb', 'ansi'],
-            help="select a terminal emulator to interpret the output (default to 'dumb').")
+        "+term",
+        default='dumb',
+        choices=['as-is', 'dumb', 'ansi'],
+        help=
+        "select a terminal emulator to interpret the output (default to 'dumb')."
+    )
+    options_parser.add_flag(
+        "input", default=False, help="enable the input tags [...]"
+    )
+
+    options_parser.add_argument(
+        "+input-prefix-range",
+        default=(6, 12),
+        type=_range,
+        help=
+        "amount of characters that must precede at minimum/maximum an input tag in the form min:max"
+    )
 
     return options_parser
 
 
+@log_context('byexample.options')
 def get_options(args, cfg):
     # the options object should have a set of default values based
     # on the flags and values from the command line.
-    options = Options({
-                        'interact': False,
-                        'shebangs': args.shebangs,
-                        })
-    log("Options (cmdline): %s" % options, cfg['verbosity']-2)
+    options = Options(
+        {
+            'interact': False,
+            'shebangs': args.shebangs,
+            'difftool': args.difftool,
+        }
+    )
+    clog().chat("Options (cmdline): %s", options)
 
     # create a parser for the rest of the options.
     optparser = get_default_options_parser(args)
     options['optparser'] = optparser
     options.update(optparser.defaults())
 
-    log("Options (cmdline + byexample's defaults): %s" % options, cfg['verbosity']-2)
+    clog().chat("Options (cmdline + byexample's defaults): %s", options)
     # we parse the argument 'options' to allow the user to change
     # some options.
     #
@@ -252,9 +310,11 @@ def get_options(args, cfg):
     #
     # Because this, we pass these to the rest of the system to be used and
     # completed later
-    cfg['options']= options
+    cfg['options'] = options
 
-    log("Options (cmdline + byexample's defaults + --options): %s" % options, cfg['verbosity']-2)
+    clog().chat(
+        "Options (cmdline + byexample's defaults + --options): %s", options
+    )
 
     options['x'] = {}
     for k, v in vars(args).items():
@@ -263,9 +323,12 @@ def get_options(args, cfg):
 
     return options
 
+
 def show_options(cfg, registry, allowed_languages):
-    parsers = [p for p in registry['parsers'].values()
-               if p.language in allowed_languages]
+    parsers = [
+        p for p in registry['parsers'].values()
+        if p.language in allowed_languages
+    ]
 
     concerns = [c for c in registry['concerns'].values()]
 
@@ -278,11 +341,18 @@ def show_options(cfg, registry, allowed_languages):
 
     for concern in concerns:
         _title("%s's specific options" % concern.target)
-        concern.get_extended_option_parser(parent_parser=None).print_help()
+        with enhance_exceptions(
+            'Extending the options', concern, cfg['use_colors']
+        ):
+            concern.get_extended_option_parser(parent_parser=None).print_help()
 
     for parser in parsers:
         _title("%s's specific options" % parser.language)
-        parser.get_extended_option_parser(parent_parser=None).print_help()
+        with enhance_exceptions(
+            'Extending the options', parser, cfg['use_colors']
+        ):
+            parser.get_extended_option_parser(parent_parser=None).print_help()
+
 
 def extend_option_parser_with_concerns(cfg, registry):
     concerns = [c for c in registry['concerns'].values()]
@@ -291,47 +361,101 @@ def extend_option_parser_with_concerns(cfg, registry):
     # starting from the byexample's one
     optparser = cfg['options']['optparser']
     for concern in concerns:
-        # update the options with the concerns's default ones and
-        # only with them; only after this merge concerns's parser with
-        # the previous parser
-        concern_optparser = concern.get_extended_option_parser(parent_parser=None)
-        cfg['options'].update(concern_optparser.defaults())
+        with enhance_exceptions(
+            'Extending the options', concern, cfg['use_colors']
+        ):
+            # update the options with the concerns's default ones and
+            # only with them; only after this merge concerns's parser with
+            # the previous parser
+            concern_optparser = concern.get_extended_option_parser(
+                parent_parser=None
+            )
+            cfg['options'].update(concern_optparser.defaults())
 
-        optparser = concern.get_extended_option_parser(optparser)
+            optparser = concern.get_extended_option_parser(optparser)
 
     cfg['options']['optparser'] = optparser
     return optparser
 
+
+@log_context('byexample.options')
 def extend_options_with_language_specific(cfg, registry, allowed_languages):
-    parsers = [p for p in registry['parsers'].values()
-               if p.language in allowed_languages]
+    parsers = [
+        p for p in registry['parsers'].values()
+        if p.language in allowed_languages
+    ]
 
     # update the defaults for all the parsers
     for parser in parsers:
-        parser_optparser = parser.get_extended_option_parser(parent_parser=None)
-        cfg['options'].update(parser_optparser.defaults())
+        with enhance_exceptions(
+            'Extending the options', parser, cfg['use_colors']
+        ):
+            parser_optparser = parser.get_extended_option_parser(
+                parent_parser=None
+            )
+            cfg['options'].update(parser_optparser.defaults())
 
     # update again but with the options that *may* had set in the command line
     for parser in parsers:
         opts = parser.extract_cmdline_options(cfg['opts_from_cmdline'])
         cfg['options'].update(opts)
 
-    log("Options (cmdline + --options + language specific): %s" % cfg['options'],
-            cfg['verbosity']-2)
+    clog().chat(
+        "Options (cmdline + --options + language specific): %s", cfg['options']
+    )
 
+
+def verbosity_to_log_levels(verbosity, quiet):
+    if quiet:
+        return {'byexample': CRITICAL}
+
+    tmp = [
+        {
+            'byexample': NOTE
+        },
+        {
+            'byexample': NOTE,
+            'byexample.exec': INFO
+        },  # -v
+        {
+            'byexample': NOTE,
+            'byexample.exec': CHAT
+        },  # -vv
+        {
+            'byexample': INFO,
+            'byexample.exec': CHAT
+        },  # -vvv
+        {
+            'byexample': CHAT
+        },  # -vvvv
+        {
+            'byexample': DEBUG
+        },  # -vvvvv
+        {
+            'byexample': TRACE
+        },  # -vvvvvv
+    ]
+
+    return tmp[min(verbosity, len(tmp) - 1)]
+
+
+@log_context('byexample.init')
 def init(args):
-    verify_encodings(args.encoding, args.verbosity)
+    lvl = verbosity_to_log_levels(args.verbosity, args.quiet)
+    lvl.update(args.log_masks)
+    setLogLevels(lvl)
 
+    verify_encodings(args.encoding, args.verbosity)
     cfg = {
-            'use_progress_bar': args.pretty == 'all',
-            'use_colors': args.pretty == 'all',
-            'quiet':      args.quiet,
-            'verbosity':  args.verbosity,
-            'encoding':   args.encoding,
-            'output':     sys.stdout,
-            'interact':   False,
-            'opts_from_cmdline': args.options_str,
-            }
+        'use_progress_bar': args.pretty == 'all',
+        'use_colors': args.pretty == 'all',
+        'quiet': args.quiet,
+        'verbosity': args.verbosity,
+        'encoding': args.encoding,
+        'output': sys.stdout,
+        'interact': False,
+        'opts_from_cmdline': args.options_str,
+    }
 
     allowed_files = set(args.files) - set(args.skip)
     testfiles = list(sorted(f for f in args.files if f in allowed_files))
@@ -354,9 +478,10 @@ def init(args):
 
     if not testfiles:
         if not cfg['quiet']:
-            log("No files were found (you passed %i files and %i were skipped)" %
-                    (len(set(args.files)), len(set(args.files) - allowed_files)),
-                    cfg['verbosity'])
+            clog().error(
+                "No files were found (you passed %i files and %i were skipped)",
+                (len(set(args.files)), len(set(args.files) - allowed_files))
+            )
         sys.exit(1)
 
     # extend the option parser with all the parsers of the concerns.
@@ -371,14 +496,18 @@ def init(args):
     if cfg['quiet']:
         registry['concerns'].pop('progress', None)
 
-    log("Configuration:\n%s." % pprint.pformat(cfg), cfg['verbosity']-2)
-    log("Registry:\n%s." % pprint.pformat(registry), cfg['verbosity']-2)
+    if not clog().isEnabledFor(CHAT):
+        clog().chat("Options:\n%s.", pprint.pformat(cfg['options']))
+
+    clog().chat("Configuration:\n%s.", pprint.pformat(cfg))
+    clog().chat("Registry:\n%s.", pprint.pformat(registry))
 
     concerns = ConcernComposite(registry, **cfg)
 
     differ = Differ(**cfg)
 
     harvester = ExampleHarvest(allowed_languages, registry, **cfg)
-    executor  = FileExecutor(concerns, differ, **cfg)
+    executor = FileExecutor(concerns, differ, **cfg)
 
+    configure_log_system(use_colors=cfg['use_colors'], concerns=concerns)
     return testfiles, harvester, executor, options
