@@ -1,4 +1,4 @@
-"""
+r"""
 Example:
   ?: #include <iostream>
 
@@ -14,23 +14,38 @@ Example:
   ?: j + 3
   (int) 11
 
+  ?: std::cout << "this\n" \
+  :: "is a multiline\n"    \
+  :: "string\n";
+  this
+  is a multiline
+  string
+
+  ?: /* this
+  :: is a multiline
+  :: comment */
+
+  ?: std::cout << "okay\n";
+  okay
 """
 
 from __future__ import unicode_literals
 import re, sys, time
 from byexample.common import constant
 from byexample.parser import ExampleParser
-from byexample.runner import ExampleRunner, PexepctMixin, ShebangTemplate
+from byexample.runner import ExampleRunner, PexpectMixin, ShebangTemplate
 from byexample.finder import ExampleFinder
 
 stability = 'experimental'
 
+
 class CppPromptFinder(ExampleFinder):
     target = 'cpp-prompt'
 
     @constant
     def example_regex(self):
-        return re.compile(r'''
+        return re.compile(
+            r'''
             (?P<snippet>
                 (?:^(?P<indent> [ ]*) (?:\?:)[ ]  .*)         # PS1 line
                 (?:\n           [ ]*  ::             .*)*)    # PS2 lines
@@ -40,13 +55,16 @@ class CppPromptFinder(ExampleFinder):
                           (?![ ]*(?:\?:))     # Not a line starting with PS1
                           .+$\n?              # But any other line
                       )*)
-            ''', re.MULTILINE | re.VERBOSE)
+            ''', re.MULTILINE | re.VERBOSE
+        )
 
     def get_language_of(self, *args, **kargs):
         return 'cpp'
 
     def get_snippet_and_expected(self, match, where):
-        snippet, expected = ExampleFinder.get_snippet_and_expected(self, match, where)
+        snippet, expected = ExampleFinder.get_snippet_and_expected(
+            self, match, where
+        )
 
         snippet = self._remove_prompts(snippet)
         return snippet, expected
@@ -55,6 +73,7 @@ class CppPromptFinder(ExampleFinder):
         lines = snippet.split("\n")
         return '\n'.join(line[3:] for line in lines)
 
+
 class CPPParser(ExampleParser):
     language = 'cpp'
 
@@ -62,28 +81,30 @@ class CPPParser(ExampleParser):
     def example_options_string_regex(self):
         # anything of the form:
         #   //  byexample:  +FOO -BAR +ZAZ=42
-        return re.compile(r'//\s*byexample:\s*([^\n\'"]*)$',
-                                                    re.MULTILINE)
+        return re.compile(r'//\s*byexample:\s*([^\n\'"]*)$', re.MULTILINE)
 
-class CPPInterpreter(ExampleRunner, PexepctMixin):
+
+class CPPInterpreter(ExampleRunner, PexpectMixin):
     language = 'cpp'
 
     def __init__(self, verbosity, encoding, **unused):
         self.encoding = encoding
 
-        PexepctMixin.__init__(self,
-                                PS1_re =    r'\[cling\]\$',           # [cling]$
-                                any_PS_re = r'\[cling\][$!](?: \?)?') # [cling]!
-                                                                      # [cling]$ ?
+        PexpectMixin.__init__(
+            self,
+            PS1_re=r'\[cling\]\$',  # [cling]$
+            any_PS_re=r'\[cling\][$!](?: \?)?'
+        )  # [cling]!
+        # [cling]$ ?
 
     def get_default_cmd(self, *args, **kargs):
-        return  "%e %p %a", {
-                    'e': "/usr/bin/env",
-                    'p': "cling",
-                    'a': [
-                            "--nologo", # do not output the banner
-                        ]
-                    }
+        return "%e %p %a", {
+            'e': "/usr/bin/env",
+            'p': "cling",
+            'a': [
+                "--nologo",  # do not output the banner
+            ]
+        }
 
     def run(self, example, options):
         # the algorithm to filter the echos from the cling's output
@@ -95,13 +116,15 @@ class CPPInterpreter(ExampleRunner, PexepctMixin):
         # so we force this (see _get_output())
         options['term'] = 'ansi'
 
-        return PexepctMixin._run(self, example, options)
+        return PexpectMixin._run(self, example, options)
 
     def _run_impl(self, example, options):
-        return self._exec_and_wait(example.source, options)
+        return self._exec_and_wait(
+            example.source, options, from_example=example
+        )
 
     def interact(self, example, options):
-        PexepctMixin.interact(self)
+        PexpectMixin.interact(self)
 
     def initialize(self, options):
         shebang, tokens = self.get_default_cmd()
@@ -110,11 +133,12 @@ class CPPInterpreter(ExampleRunner, PexepctMixin):
         cmd = ShebangTemplate(shebang).quote_and_substitute(tokens)
 
         options.up()
-        options['geometry'] = (max(options['geometry'][0], 128), max(options['geometry'][1], 128))
+        options['geometry'] = (
+            max(options['geometry'][0], 128), max(options['geometry'][1], 128)
+        )
         self._spawn_interpreter(cmd, options)
         options.down()
 
-
     def _change_terminal_geometry(self, rows, cols, options):
         raise Exception("This should never happen")
 
@@ -122,27 +146,7 @@ class CPPInterpreter(ExampleRunner, PexepctMixin):
         self._shutdown_interpreter()
 
     def _get_output(self, options):
-        # cling doesn't disable the TTY's echo so everything we type in
-        # it will be reflected in the output.
-        # so this breaks badly self._get_output
-
-        # self.last_output is a list of strings found by pexpect
-        # after returning of each pexpect.expect
-        # in other words if we prefix each line with the prompt
-        # should get the original output from the process
-        lines = ('[cling]$ ' + line for line in self.last_output)
-        self._drop_output()
-
-        # now, feed those lines to our ANSI Terminal emulator
-        lines = self._emulate_ansi_terminal(lines, join=False)
-
-        # get each line in the Terminal's display and ignore each one that
-        # belong with our prompt: those are the "echo" lines that
-        # *we* sent to the cling and they are not part of *its* output.
-        lines = (line for line in lines
-                               if not line.startswith('[cling]$'))
-
-        return '\n'.join(lines)
+        return self._get_output_echo_filtered(options)
 
     def cancel(self, example, options):
-        return False    # not supported by cling
+        return False  # not supported by cling
