@@ -1,25 +1,37 @@
 from __future__ import unicode_literals
-import re, shlex, argparse
-from .common import log, tohuman, constant
+import re, shlex, argparse, bisect, collections
+from .common import tohuman, constant
 from .options import OptionParser, UnrecognizedOption, ExtendOptionParserMixin
 from .expected import _LinearExpected, _RegexExpected
 from .parser_sm import SM_NormWS, SM_NotNormWS
+'''
+>>> from byexample.log import init_log_system
+>>> init_log_system()
+'''
+
 
 def tag_name_as_regex_name(name):
     return name.replace('-', '_')
 
+
+TagRegexs = collections.namedtuple('TagRegexs', ['for_split', 'for_capture'])
+InputRegexs = collections.namedtuple(
+    'InputRegexs', ['for_check', 'for_capture']
+)
+
+
 class ExampleParser(ExtendOptionParserMixin):
     def __init__(self, verbosity, encoding, options, **unused):
         ExtendOptionParserMixin.__init__(self)
         self.verbosity = verbosity
-        self.encoding  = encoding
-        self.options   = options
+        self.encoding = encoding
+        self.options = options
 
         self._optparser_extended_cache = None
         self._opts_cache = {}
 
     def __repr__(self):
-        return '%s Parser' % tohuman(self.language)
+        return '%s Parser' % tohuman(self.language if self.language else self)
 
     def example_options_string_regex(self):
         '''
@@ -35,7 +47,7 @@ class ExampleParser(ExtendOptionParserMixin):
           # ~byexample~ bla bla
 
         '''
-        raise NotImplementedError() # pragma: no cover
+        raise NotImplementedError()  # pragma: no cover
 
     def example_options_as_list(self, string):
         '''
@@ -56,18 +68,51 @@ class ExampleParser(ExtendOptionParserMixin):
         return parser
 
     @constant
-    def capture_tag_regex(self):
+    def capture_tag_regexs(self):
         '''
-        Return a regular expression to match a 'capture tag'.
+        Return a set of regular expressions to match a 'capture tag'.
 
         Due implementation details the underscore character '_'
         *cannot* be used as a valid character in the name.
         Instead you should use minus '-'.
+
+        The returned regex can be used for splitting a string
+        or for capturing.
         '''
-        return {
-                'split': re.compile(r"(<(?:[^\W_]|-|\.)+>)"),
-                'full': re.compile(r"<(?P<name>(?:[^\W_]|-|\.)+)>"),
-                }
+        open, close = map(re.escape, '<>')
+
+        name_re = r'[A-Za-z.][A-Za-z0-9:.-]*'
+        return TagRegexs(
+            for_split=re.compile(r"(%s%s%s)" % (open, name_re, close)),
+            for_capture=re.compile(
+                r"%s(?P<name>%s)%s" % (open, name_re, close)
+            )
+        )
+
+    @constant
+    def input_regexs(self):
+        open, close = map(re.escape, '[]')
+        input_re = r'''
+            %s          # open marker
+            (?P<input>
+            [^%s\\]*    # neither a close marker or a slash
+            (?:\\.[^%s\\]*)*    # a "escaped" char followed by
+                                # 0 or more "neither a close marker or a slash"
+            )
+            %s          # a close marker
+            ''' % (open, close, close, close)
+
+        input_re_at_end = r'''
+            %s          # the input regex
+            (?P<trailing>
+                [ ]*$   # followed by some optional space and a end of line
+            )
+            ''' % (input_re)
+
+        return InputRegexs(
+            for_check=re.compile(input_re, re.VERBOSE | re.MULTILINE),
+            for_capture=re.compile(input_re_at_end, re.VERBOSE | re.MULTILINE)
+        )
 
     def ellipsis_marker(self):
         return '...'
@@ -84,12 +129,12 @@ class ExampleParser(ExtendOptionParserMixin):
         '''
 
         if not expected:
-            expected = ''   # make sure that it is an empty string
+            expected = ''  # make sure that it is an empty string
 
         if not snippet.endswith('\n'):
-            snippet += '\n' # make sure that we end the code with a newline
-                            # most of the runners use this to flush and
-                            # execute the code
+            snippet += '\n'  # make sure that we end the code with a newline
+            # most of the runners use this to flush and
+            # execute the code
 
         return snippet, expected
 
@@ -100,8 +145,8 @@ class ExampleParser(ExtendOptionParserMixin):
         options.up(local_options)
 
         example.source, example.expected_str = self.process_snippet_and_expected(
-                                                              example.snippet,
-                                                              example.expected_str)
+            example.snippet, example.expected_str
+        )
 
         # the options to customize this example
         example.options = local_options
@@ -112,42 +157,48 @@ class ExampleParser(ExtendOptionParserMixin):
         for x in options['rm']:
             example.expected_str = example.expected_str.replace(x, '')
 
-        expected_regexs, charnos, rcounts, tags_by_idx = self.expected_as_regexs(
-                                                example.expected_str,
-                                                options['tags'],
-                                                options['norm_ws'])
+        input_prefix_len_range = options['input_prefix_range']
+        expected_regexs, charnos, rcounts, tags_by_idx, input_list = self.expected_as_regexs(
+            example.expected_str, options['tags'], options['input'],
+            options['norm_ws'], input_prefix_len_range
+        )
 
         ExpectedClass = _LinearExpected
 
         expected = ExpectedClass(
-                          # the output expected
-                          expected_str=example.expected_str,
+            # the output expected
+            expected_str=example.expected_str,
 
-                          # expected regex version
-                          regexs=list(expected_regexs),
+            # expected regex version
+            regexs=list(expected_regexs),
 
-                          # where each regex comes from
-                          charnos=list(charnos),
+            # where each regex comes from
+            charnos=list(charnos),
 
-                          # the 'real count' of literals
-                          rcounts=list(rcounts),
+            # the 'real count' of literals
+            rcounts=list(rcounts),
 
-                          # all the regexs that are not literal (tags) indexed
-                          # by their position in the regex list.
-                          # we don't save the regex (use 'regexs' for that),
-                          # instead save the name of the tag or None if it's
-                          # unnamed
-                          tags_by_idx=tags_by_idx,
-                          )
+            # all the regexs that are not literal (tags) indexed
+            # by their position in the regex list.
+            # we don't save the regex (use 'regexs' for that),
+            # instead save the name of the tag or None if it's
+            # unnamed
+            tags_by_idx=tags_by_idx,
+        )
 
         # the source code to execute and the expected
         example.expected = expected
 
+        # the things that we need to type when we run the example
+        example.input_list = input_list
+
         options.down()
         return example
 
-
-    def expected_as_regexs(self, expected, tags_enabled, normalize_whitespace):
+    def expected_as_regexs(
+        self, expected, tags_enabled, input_enabled, normalize_whitespace,
+        input_prefix_len_range
+    ):
         '''
         From the expected string create a list of regular expressions that
         joined with the flags re.MULTILINE | re.DOTALL, matches
@@ -161,15 +212,19 @@ class ExampleParser(ExtendOptionParserMixin):
             - a dict of non-literal 'regexs' names (capturing and non-capturing)
               also know as "tags" indexed by position.
               For non-capturing the name will be None.
+            - a list of (<prefix>, <input>) tuples that describe, in order, the
+              text to type (input) and the text to wait for before the
+              typing (prefix)
 
             >>> from byexample.parser import ExampleParser
+            >>> from functools import partial
             >>> import re
 
             >>> parser = ExampleParser(0, 'utf8', None); parser.language = 'python'
-            >>> _as_regexs = parser.expected_as_regexs
+            >>> _as_regexs = partial(parser.expected_as_regexs, tags_enabled=True, input_enabled=True, normalize_whitespace=False, input_prefix_len_range=(6,12))
 
             >>> expected = 'a<foo>b<bar>c'
-            >>> regexs, charnos, rcounts, tags_by_idx = _as_regexs(expected, True, False)
+            >>> regexs, charnos, rcounts, tags_by_idx, input_list = _as_regexs(expected)
 
         We return the regexs
 
@@ -208,7 +263,7 @@ class ExampleParser(ExtendOptionParserMixin):
         we enable the normalization of the whitespace:
 
             >>> expected = 'a<...> <foo-bar>c'
-            >>> regexs, _, _, tags_by_idx = _as_regexs(expected, True, True)
+            >>> regexs, _, _, tags_by_idx, _ = _as_regexs(expected, normalize_whitespace=True)
 
             >>> regexs          # byexample: +norm-ws
             ('\\A', 'a', '(?:.*?)(?<!\\s)', '\\s+(?!\\s)', '(?P<foo_bar>.*?)', 'c', '\\s*\\Z')
@@ -216,15 +271,18 @@ class ExampleParser(ExtendOptionParserMixin):
             >>> tags_by_idx
             {2: None, 4: 'foo-bar'}
         '''
-        capture_tag_regex = self.capture_tag_regex()['full']
-        capture_tag_split_regex = self.capture_tag_regex()['split']
-        ellipsis_marker = self.ellipsis_marker()
         if normalize_whitespace:
-            sm = SM_NormWS(capture_tag_regex, capture_tag_split_regex, ellipsis_marker)
+            sm = SM_NormWS(
+                self.capture_tag_regexs(), self.input_regexs(),
+                self.ellipsis_marker(), input_prefix_len_range
+            )
         else:
-            sm = SM_NotNormWS(capture_tag_regex, capture_tag_split_regex, ellipsis_marker)
+            sm = SM_NotNormWS(
+                self.capture_tag_regexs(), self.input_regexs(),
+                self.ellipsis_marker(), input_prefix_len_range
+            )
 
-        return sm.parse(expected, tags_enabled)
+        return sm.parse(expected, tags_enabled, input_enabled)
 
     def extract_cmdline_options(self, opts_from_cmdline):
         # now we can re-parse this argument 'options' from the command line
@@ -247,9 +305,14 @@ class ExampleParser(ExtendOptionParserMixin):
             optlist = self.example_options_as_list(optstring_match.group(1))
 
         if not isinstance(optlist, list):
-            raise ValueError("The option list returned by the parser is not a list!. This probably means that there is a bug in the parser %s." % str(self))
+            raise ValueError(
+                "The option list returned by the parser is not a list!. This probably means that there is a bug in the parser %s."
+                % str(self)
+            )
 
-        return self._extend_parser_and_parse_options_strictly_and_cache(optlist)
+        return self._extend_parser_and_parse_options_strictly_and_cache(
+            optlist
+        )
 
     def _extend_parser_and_parse_options_strictly(self, optlist):
         # we parse the example's options
@@ -261,7 +324,7 @@ class ExampleParser(ExtendOptionParserMixin):
         optparser = self.options['optparser']
         optparser_extended = self.get_extended_option_parser(optparser)
         try:
-          opts = optparser_extended.parse(optlist, strict=True)
+            opts = optparser_extended.parse(optlist, strict=True)
         except UnrecognizedOption as e:
             raise ValueError(str(e))
 
@@ -294,10 +357,11 @@ class ExampleParser(ExtendOptionParserMixin):
             self._opts_cache[tuple(optlist)] = val
             return val
 
+
 # Extra tests
 '''
 >>> expected = 'ex <...>\nu<...>'
->>> regexs, _, _, _ = _as_regexs(expected, True, True)
+>>> regexs, _, _, _, _ = _as_regexs(expected, normalize_whitespace=True)
 
 >>> regexs
 ('\\A',
@@ -314,7 +378,7 @@ class ExampleParser(ExtendOptionParserMixin):
 ()
 
 >>> expected = 'ex <foo>\nu<bar>'
->>> regexs, _, _, _ = _as_regexs(expected, True, True)
+>>> regexs, _, _, _, _ = _as_regexs(expected, normalize_whitespace=True)
 
 >>> regexs
 ('\\A',
